// --------------------------------------------------------
#include "IPersistObject.h"
#include "CPersistFactory.h"
// --------------------------------------------------------
IPersistObject::IPersistObject() : MAllocType()
{
  m_iId=-1;
  m_Factory=CPersistFactory::getInstance();
  if (isDynamic())
  {
    // the object is created by the user with new - register it in factory as object
    m_Factory->registerObject(this);
  }
  else
  {
    // the object is created by the user without new - register it in factory as data
    m_Factory->registerData(this);
  }
# ifdef PERSIST_DEBUG
  if (isDynamic())
    cerr << "Dynamic object " << m_iId << endl;
  else
    cerr << "Static object " << m_iId << endl;
# endif
}
// --------------------------------------------------------
IPersistObject::IPersistObject(bool dynamic) : MAllocType()
{
  m_iId=-1;
  m_Factory=CPersistFactory::getInstance();
  forceDynamic(dynamic);
  if (dynamic)
  {
    // the object is created by the user with new - register it in factory as object
    m_Factory->registerObject(this);
  }
  else
  {
    m_Factory->registerData(this);
  } 
}
// --------------------------------------------------------
IPersistObject::IPersistObject(istream& s)
{
  m_iId=-1;
  m_Factory=CPersistFactory::getInstance();
  // the object is loaded by factory - read it from stream
  /*
    load(s); -> can't work : virtual call in constructor ! 
    PERSIST macro is used to build the correct constructor in the child object.
  */
}
// --------------------------------------------------------
IPersistObject::~IPersistObject()
{
  if (idDefined())
  {
    if (isDynamic())
      m_Factory->unregisterObject(this);
    else
      m_Factory->unregisterData(this);
  }
}
// --------------------------------------------------------
