/**
   \class IPersistType

   \brief Interface for describing persistent types

*/
// --------------------------------------------------------
//  Author : Sylvain Lefebvre
//  History: Created ... 29/12/2001
// --------------------------------------------------------
#ifndef __PERSIST_TYPE__
#define __PERSIST_TYPE__
// --------------------------------------------------------
class CPersistFactory;
class IPersistObject;
// --------------------------------------------------------
#ifdef WIN32
#include <istream>
#else
#include <iostream>
#endif
// --------------------------------------------------------

using namespace std;
// --------------------------------------------------------

class IPersistType
{
 protected:
  const char *m_szTypeId;

  void        autoRegister();

  friend class CPersistFactory;
 public:
  IPersistType(const char *s) { m_szTypeId=s; }
  virtual ~IPersistType() {}
  virtual IPersistObject *operator() (istream&)=0;
  virtual IPersistObject *operator() ()=0;
};
// --------------------------------------------------------
template <class T> class CPersistType : public IPersistType
{
public:
  CPersistType(const char *s) : IPersistType(s)        { autoRegister(); }

  virtual IPersistObject *operator()   (istream& s) { return (new T(s)); }
  virtual IPersistObject *operator()   () { return (new T()); }
};
// --------------------------------------------------------
#define PERSIST_ID(a) CPersistType<a> *a::s_PersistType = new CPersistType<a>(#a);
// --------------------------------------------------------
#define PERSIST_SUBCLASS_ID(a,b) CPersistType<b> *a::b::s_PersistType = new CPersistType<b>();
// --------------------------------------------------------
#ifdef PERSIST_XML
#define PERSIST_LOAD_SAVE_MEMBER_PROTOTYPE virtual list<CPersistFactory::CPtrStock *> xmlPersistLoadSaveMembers(const char *,vector<string>&,ostream *);
#else
#define PERSIST_LOAD_SAVE_MEMBER_PROTOTYPE virtual void rawPersistLoadSaveMembers(istream *,ostream *);
#endif
// --------------------------------------------------------
//! This macro are used to add special stuff in persistent objects
#define PERSIST_OBJECT(a) \
                          public: \
                             a(istream& s) : IPersistObject(s) {init(); load(s);} \
                             static CPersistType<a> *s_PersistType; \
                             virtual IPersistType *getPersistType() {return (a::s_PersistType);} \
	       	          protected: \
                             PERSIST_LOAD_SAVE_MEMBER_PROTOTYPE \
		          public:
// --------------------------------------------------------
#define PERSIST_OBJECT_INHERITS(a,b) \
                          public:  \
                             a(istream& s) : b() {init(); load(s);} \
                             static CPersistType<a> *s_PersistType; \
                             virtual IPersistType *getPersistType() {return (a::s_PersistType);} \
	              	  protected: \
                             PERSIST_LOAD_SAVE_MEMBER_PROTOTYPE \
	           	  public:
// --------------------------------------------------------
#define PERSIST_DATA(a) \
                          public: \
	       	          protected: \
                             PERSIST_LOAD_SAVE_MEMBER_PROTOTYPE \
		          public:
// --------------------------------------------------------
#define PERSIST_DATA_INHERITS(a,b) \
                          public:  \
	              	  protected: \
                             PERSIST_LOAD_SAVE_MEMBER_PROTOTYPE \
	           	  public:
// --------------------------------------------------------
#endif
// --------------------------------------------------------
