/**

   \brief Main include file

*/
// --------------------------------------------------------
//  Author : Sylvain Lefebvre
//  History: Created ... 08/02/2001
// --------------------------------------------------------
#ifndef __PERSIST__
#define __PERSIST__
// --------------------------------------------------------
#ifndef PERSIST_RAW
#define PERSIST_XML
#endif
// --------------------------------------------------------
#include <sstream>
// --------------------------------------------------------
#include "IPersistType.h"
#include "IPersistObject.h"
#include "CPersistFactory.h"
// --------------------------------------------------------
#include "persist_string.h"
// --------------------------------------------------------
// Macros load/save
// --------------------------------------------------------
#define PERSIST_LOAD_RAW(f) CPersistFactory::getInstance()->rawLoad(f)
// --------------------------------------------------------
#define PERSIST_SAVE_RAW(f) CPersistFactory::getInstance()->rawSave(f)
// --------------------------------------------------------
#define PERSIST_LOAD_XML(f) CPersistFactory::getInstance()->xmlLoad(f)
// --------------------------------------------------------
#define PERSIST_SAVE_XML(f) CPersistFactory::getInstance()->xmlSave(f)
// --------------------------------------------------------
#define PERSIST_SAVE_INSTANCE_RAW(o,f) CPersistFactory::getInstance()->rawSaveInstance(o,f)
// --------------------------------------------------------
#define PERSIST_SAVE_INSTANCE_XML(o,f) CPersistFactory::getInstance()->xmlSaveInstance(o,f)
// --------------------------------------------------------
#define PERSIST_LOAD_INSTANCE_RAW(t,f) dynamic_cast<t *>(CPersistFactory::getInstance()->rawLoadInstance(f));
// --------------------------------------------------------
#define PERSIST_LOAD_INSTANCE_XML(t,f) dynamic_cast<t *>(CPersistFactory::getInstance()->xmlLoadInstance(f));
// --------------------------------------------------------
#define PERSIST_TRY_LOAD_RAW(f) try {CPersistFactory::getInstance()->rawLoad(f);} catch (CException& ) {cerr << "PERSIST_TRY_LOAD: No data." << endl;}
// --------------------------------------------------------
#define PERSIST_TRY_LOAD_XML(f) try {CPersistFactory::getInstance()->xmlLoad(f);} catch (CException& ) {cerr << "PERSIST_TRY_LOAD: No data." << endl;}
// --------------------------------------------------------
// Macros members
// --------------------------------------------------------
#define PERSIST_BEGIN_MEMBERS_RAW(c) void c::rawPersistLoadSaveMembers(istream *i,ostream *o) {
// --------------------------------------------------------
#define PERSIST_BEGIN_MEMBERS_XML(c) list<CPersistFactory::CPtrStock *> c::xmlPersistLoadSaveMembers(const char *n,vector<string>& i,ostream *o) { list<CPersistFactory::CPtrStock *> childs;
// --------------------------------------------------------
#define PERSIST_BEGIN_MEMBERS_INHERITS_RAW(c,h) void c::rawPersistLoadSaveMembers(istream *i,ostream *o) { h::persistLoadSaveMembers(i,o);
// --------------------------------------------------------
#define PERSIST_BEGIN_MEMBERS_INHERITS_XML(c,h) list<CPersistFactory::CPtrStock *> c::xmlPersistLoadSaveMembers(const char *n,vector<string>& i,ostream *o) { list<CPersistFactory::CPtrStock *> childs=h::xmlPersistLoadSaveMembers(n,i,o);
// --------------------------------------------------------
#define PERSIST_MEMBER_RAW(p)     if (o != NULL) *o << p << ' '; \
                                  if (i != NULL) *i >> p;
// --------------------------------------------------------
#define PERSIST_MEMBER_XML(p)     if (o != NULL) \
				    *o << "    <member name=\"" << #p << "\">" << p << "</member>" << endl; \
                                  else if (!i.empty()) \
                                  { \
				    if (!strcmp(#p,n)) \
                                    { \
                                      istringstream in(i[0]);\
                                      in >> p; \
                                      \
                                    }\
				  }
// --------------------------------------------------------
#define PERSIST_SEQUENCE_RAW(C,T,p) \
  if (o != NULL) \
  { \
    *o << p.size() << ' '; \
    for (C<T>::iterator I=p.begin();I!=p.end();I++) \
    { \
       *o << (*I) << ' '; \
    } \
  } \
  else if (i != NULL) \
  { \
    unsigned int __i__,__n__; \
    *i >> __n__; \
    for (__i__=0;__i__<__n__;__i__++) \
    { \
       p.insert(p.end(),T()); \
       *i >> (*(p.end()-1)); \
    } \
  }
// --------------------------------------------------------
#define PERSIST_SEQUENCE_XML(C,T,p) \
  if (o != NULL) \
  { \
    *o << "    <sequence name=\"" << #p << "\" size=\"" << p.size() << "\">" << endl; \
    for (C<T>::iterator I=p.begin();I!=p.end();I++) \
    { \
       *o << "      <element>" << (*I) << "</element>" << endl; \
    } \
    *o << "    </sequence>" << endl; \
  } \
  else if (!i.empty()) \
  { \
    if (!strcmp(#p,n)) \
    {\
      unsigned int __i__; \
      for (__i__=0;__i__<i.size();__i__++) \
      { \
        istringstream in(i[__i__]);\
        p.insert(p.end(),T()); \
        in >> (*(p.end()-1)); \
      } \
    } \
  }
// --------------------------------------------------------
#define PERSIST_TABLE_RAW(p,s,e) \
  if (o != NULL) \
  { \
    for (unsigned int __i__=s;__i__<=e;__i__++) \
    { \
       *o << p[__i__] << ' ';\
    } \
  } \
  if (i != NULL) \
  { \
    for (unsigned int __i__=s;__i__<=e;__i__++) \
    { \
       *i >> p[__i__];\
    } \
  }
// --------------------------------------------------------
#define PERSIST_TABLE_XML(p,s,e) \
  if (o != NULL) \
  { \
    *o << "    <sequence name=\"" << #p << "\" size=\"" << (e-s+1) << "\">" << endl; \
    for (unsigned int __i__=s;__i__<=e;__i__++) \
    { \
       *o << "      <element>" << p[__i__] << "</element>" << endl; \
    } \
    *o << "    </sequence>" << endl; \
  } \
  else if (!i.empty()) \
  { \
    if (!strcmp(#p,n)) \
    {\
      unsigned int __i__; \
      for (__i__=s;__i__<=e;__i__++) \
      { \
        istringstream in(i[__i__]);\
        in >> p[__i__]; \
      } \
    } \
  }
// --------------------------------------------------------
#define PERSIST_PTR_MEMBER_RAW(T,p)   if (o != NULL) m_Factory->rawSaveChild(*o,dynamic_cast<IPersistObject *>(p)); \
                                      if (i != NULL) m_Factory->rawLoadChild(*i,new CPersistFactory::CPtrHack<T>(&p));
// --------------------------------------------------------
#define PERSIST_PTR_MEMBER_XML(T,p)   if (o != NULL) \
                                      { \
					*o << "    <ptr name=\"" << #p << "\""; \
                                        m_Factory->xmlSaveChild(*o,dynamic_cast<IPersistObject *>(p)); \
					*o << "/>" << endl; \
                                      } \
                                      else if (!strcmp(#p,n)) childs.push_back(new CPersistFactory::CPtrHack<T>(&p));
// --------------------------------------------------------
#define PERSIST_PTR_SEQUENCE_RAW(C,T,p) \
  if (o != NULL) \
  { \
    *o << p.size() << ' '; \
    for (C<T *>::iterator I=p.begin();I!=p.end();I++) \
    { \
       m_Factory->rawSaveChild(*o,dynamic_cast<IPersistObject *>((*I))); \
    } \
  } \
  if (i != NULL) \
  { \
    unsigned int __i__,__n__; \
    *i >> __n__; \
    p.clear(); \
    for (__i__=0;__i__<__n__;__i__++) \
    { \
       p.insert(p.end(),NULL); \
    } \
    for (C<T *>::iterator I=p.begin();I!=p.end();I++) \
    { \
       m_Factory->rawLoadChild(*i,new CPersistFactory::CPtrHack<T>(&(*I))); \
    } \
  }
// --------------------------------------------------------
#define PERSIST_PTR_SEQUENCE_XML(C,T,p) \
  if (o != NULL) \
  { \
    *o << "    <ptrsequence name=\"" << #p << "\" size=\"" << p.size() << "\">" << endl; \
    for (C<T *>::iterator I=p.begin();I!=p.end();I++) \
    { \
       *o << "      <ptrelement "; \
       m_Factory->xmlSaveChild(*o,dynamic_cast<IPersistObject *>(*I)); \
       *o << "/>" << endl; \
    } \
    *o << "    </ptrsequence>" << endl; \
  } \
  if (!i.empty()) \
  { \
    if (!strcmp(#p,n)) \
    { \
      unsigned int __i__; \
      p.clear(); \
      for (__i__=0;__i__<i.size();__i__++) \
      { \
         p.insert(p.end(),NULL); \
      } \
      for (C<T *>::iterator I=p.begin();I!=p.end();I++) \
      { \
         childs.push_back(new CPersistFactory::CPtrHack<T>(&(*I))); \
      } \
    } \
  }
// --------------------------------------------------------
#define PERSIST_PTR_TABLE_RAW(T,p,s,e) \
  if (o != NULL) \
  { \
    for (unsigned int __i__=s;__i__<=e;__i__++) \
    { \
       m_Factory->rawSaveChild(*o,dynamic_cast<IPersistObject *>(p[__i__])); \
    } \
  } \
  if (i != NULL) \
  { \
    for (unsigned int __i__=s;__i__<=e;__i__++) \
    { \
       m_Factory->rawLoadChild(*i,new CPersistFactory::CPtrHack<T>(&(p[__i__])));\
    } \
  }
// --------------------------------------------------------
#define PERSIST_PTR_TABLE_XML(T,p,s,e) \
  if (o != NULL) \
  { \
    *o << "    <ptrsequence name=\"" << #p << "\" size=\"" << (e-s+1) << "\">" << endl; \
    for (unsigned int __i__=s;__i__<=e;__i__++) \
    { \
       *o << "      <ptrelement "; \
       m_Factory->xmlSaveChild(*o,dynamic_cast<IPersistObject *>(p[__i__])); \
       *o << "/>" << endl; \
    } \
    *o << "    </ptrsequence>" << endl; \
  } \
  if (!i.empty()) \
  { \
    if (!strcmp(#p,n)) \
    { \
      for (unsigned int __i__=s;__i__<=e;__i__++) \
      { \
         childs.push_back(new CPersistFactory::CPtrHack<T>(&(p[__i__]))); \
      } \
    } \
  }
// --------------------------------------------------------
#define PERSIST_END_MEMBERS_RAW }
// --------------------------------------------------------
#define PERSIST_END_MEMBERS_XML return (childs); }
// --------------------------------------------------------
// Select XML or RAW
// --------------------------------------------------------
#ifdef PERSIST_XML
#  define PERSIST_LOAD                    PERSIST_LOAD_XML
#  define PERSIST_SAVE                    PERSIST_SAVE_XML
#  define PERSIST_TRY_LOAD                PERSIST_TRY_LOAD_XML
#  define PERSIST_BEGIN_MEMBERS           PERSIST_BEGIN_MEMBERS_XML
#  define PERSIST_BEGIN_MEMBERS_INHERITS  PERSIST_BEGIN_MEMBERS_INHERITS_XML
#  define PERSIST_MEMBER                  PERSIST_MEMBER_XML
#  define PERSIST_SEQUENCE                PERSIST_SEQUENCE_XML
#  define PERSIST_TABLE                   PERSIST_TABLE_XML
#  define PERSIST_PTR_MEMBER              PERSIST_PTR_MEMBER_XML
#  define PERSIST_PTR_SEQUENCE            PERSIST_PTR_SEQUENCE_XML
#  define PERSIST_PTR_TABLE               PERSIST_PTR_TABLE_XML
#  define PERSIST_END_MEMBERS             PERSIST_END_MEMBERS_XML
#  define PERSIST_SAVE_INSTANCE           PERSIST_SAVE_INSTANCE_XML
#  define PERSIST_LOAD_INSTANCE           PERSIST_LOAD_INSTANCE_XML
#else
#  define PERSIST_LOAD                    PERSIST_LOAD_RAW
#  define PERSIST_SAVE                    PERSIST_SAVE_RAW
#  define PERSIST_TRY_LOAD                PERSIST_TRY_LOAD_RAW
#  define PERSIST_BEGIN_MEMBERS           PERSIST_BEGIN_MEMBERS_RAW
#  define PERSIST_BEGIN_MEMBERS_INHERITS  PERSIST_BEGIN_MEMBERS_INHERITS_RAW
#  define PERSIST_MEMBER                  PERSIST_MEMBER_RAW
#  define PERSIST_SEQUENCE                PERSIST_SEQUENCE_RAW
#  define PERSIST_TABLE                   PERSIST_TABLE_RAW
#  define PERSIST_PTR_MEMBER              PERSIST_PTR_MEMBER_RAW
#  define PERSIST_PTR_SEQUENCE            PERSIST_PTR_SEQUENCE_RAW
#  define PERSIST_PTR_TABLE               PERSIST_PTR_TABLE_RAW
#  define PERSIST_END_MEMBERS             PERSIST_END_MEMBERS_RAW
#  define PERSIST_SAVE_INSTANCE           PERSIST_SAVE_INSTANCE_RAW
#  define PERSIST_LOAD_INSTANCE           PERSIST_LOAD_INSTANCE_RAW
#endif
// --------------------------------------------------------
// Additionnal macros
// --------------------------------------------------------
#define PERSIST_NO_MEMBER(c) PERSIST_BEGIN_MEMBERS(c) PERSIST_END_MEMBERS
// --------------------------------------------------------
#define PERSIST_NO_MEMBER_INHERITS(c,h) PERSIST_BEGIN_MEMBERS_INHERITS(c,h) PERSIST_END_MEMBERS
// --------------------------------------------------------
#define PERSIST_AFTER_LOAD CPersistFactory::getInstance()->afterLoadPoint();
// --------------------------------------------------------
#define PERSIST_CURRENT_FILENAME CPersistFactory::getInstance()->getCurrentFileName()
// --------------------------------------------------------
#define PERSIST_ENABLE_MERGE CPersistFactory::getInstance()->setMerge(true);
// --------------------------------------------------------
#define PERSIST_DISABLE_MERGE CPersistFactory::getInstance()->setMerge(false);
// --------------------------------------------------------
#define PERSIST_APPLICATION_VERSION(a) CPersistFactory::getInstance()->setApplicationVersionString(a);
// --------------------------------------------------------
#define persist 
// --------------------------------------------------------
#endif
// --------------------------------------------------------
