// ----------------------------------------------------
#include "CInterface.h"
#include "CWorld.h"
#include "CMobile.h"
#include "CIconFactory.h"
#include "CViewer.h"
#include "messages_CInterface.h"
// ----------------------------------------------------
#define PERSIST_XML
#include <persist.h>
// ----------------------------------------------------
#include <fstream>
// ----------------------------------------------------
#ifdef QT3_SUPPORT
	#include <QPushButton>
	#include <QButtonGroup>
	#include <QColorDialog>
	#include <QComboBox>
	#include <QLabel>
	#include <QLineEdit>
	#include <QValidator>
	#include <QFileDialog>
	#include <QCheckBox>
	#include <QSpinBox>
	#include <QRegExp>
	#include <QEvent>
	#include <QSizePolicy>
	#include <QRegExp>
	#include <QMessageBox>
	#include <QStatusBar>
	#include <QWheelEvent>

	#include <Q3TextEdit>
	typedef Q3TextEdit QTextEdit;
	#include <QScrollArea>
	typedef QScrollArea QScrollView;
#else
	#include <qpushbutton.h>
	#include <qbuttongroup.h>
	#include <qcolordialog.h>
	#include <qcombobox.h>
	#include <qlabel.h>
	#include <qlineedit.h>
	#include <qvalidator.h>
	#include <qfiledialog.h>
	#include <qcheckbox.h>
	#include <qspinbox.h>
	#include <qregexp.h>
	#include <qevent.h>

	#include <qtextedit.h>
	#include <qscrollview.h>
	#include <qhbox.h>

	#include <qsizepolicy.h>
	#include <qregexp.h>
	#include <qmessagebox.h>
	#include <qstatusbar.h>
#endif
// ----------------------------------------------------
#define WHEEL_DELTA 120.0
// ----------------------------------------------------
CInterface *g_MainWindow=NULL;
// ----------------------------------------------------
std::vector<struct s_color> CInterface::s_Colors;
// ----------------------------------------------------
CInterface::CInterface(QWidget* parent, const char* name, Qt::WFlags fl, char*presetsDir, char*mobilesDir) : main_interface(parent,name,fl)
{

  m_leX->setValidator(new QDoubleValidator(m_leX));
  m_leY->setValidator(new QDoubleValidator(m_leY));
  m_leDx->setValidator(new QDoubleValidator(m_leDx));
  m_leDy->setValidator(new QDoubleValidator(m_leDy));
  m_leAgl->setValidator(new QDoubleValidator(m_leAgl));
  m_leWidth->setValidator(new QDoubleValidator(m_leWidth));
  m_leHeight->setValidator(new QDoubleValidator(m_leHeight));
  
  cout<<"presetsDir : "<<presetsDir<<endl;

  initPreset(m_sMvtPresets, m_cbMvtPresets, (std::string(presetsDir)+std::string("/mvt.presets")).c_str());
  initPreset(m_sCollidePresets, m_cbCollidePresets, (std::string(presetsDir)+std::string("/collide.presets")).c_str());
  initPreset(m_sBordLPresets, m_cbBordLPresets, (std::string(presetsDir)+std::string("/bord.presets")).c_str());
  initPreset(m_sBordRPresets, m_cbBordRPresets, (std::string(presetsDir)+std::string("/bord.presets")).c_str());
  initPreset(m_sBordTPresets, m_cbBordTPresets, (std::string(presetsDir)+std::string("/bord.presets")).c_str());
  initPreset(m_sBordBPresets, m_cbBordBPresets, (std::string(presetsDir)+std::string("/bord.presets")).c_str());
  initPreset(m_sStartPresets, m_cbStartPresets, (std::string(presetsDir)+std::string("/start.presets")).c_str());
  
  m_bSave    = true;
  m_bHotkeys = true;
  m_bPresets = true;
  // m_bNoSave=nosave;
  m_Selected=NULL;
  updateDisplay();

  if (s_Colors.empty())
  {
    s_Colors.push_back(color(0x00000000,MSG_noir));
    s_Colors.push_back(color(0xFFFFFF00,MSG_blanc));
    s_Colors.push_back(color(0xFF000000,MSG_rouge));
    s_Colors.push_back(color(0x00FF0000,MSG_vert));
    s_Colors.push_back(color(0x0000FF00,MSG_bleu));
    s_Colors.push_back(color(0xffff0000,MSG_jaune));
    s_Colors.push_back(color(0x00ffff00,MSG_cyan));
    s_Colors.push_back(color(0xff00ff00,MSG_violet));
    s_Colors.push_back(color(0xff880000,MSG_orange));
    s_Colors.push_back(color(0x80808000,MSG_gris));
    initComboColors();
  }

  //if (m_bNoSave)
  //{
  //  m_btnSaveSession->hide();
  //  m_btnSaveMobile->hide();
  //}

#ifdef QT3_SUPPORT
  m_hbVisu = new QHBox;
  m_hbVisu->setGeometry( QRect( 5, 27, 320, 40 ) );
  m_FrameVisu->setLayout(m_hbVisu);

#else
  QScrollView *scr=new QScrollView(m_FrameVisu,"scr");
  scr->setGeometry( QRect( 5, 27, 320, 40 ) );
  scr->setHScrollBarMode(QScrollView::AlwaysOn);

  m_hbVisu = new QHBox( scr->viewport(), "box" );
  scr->addChild(m_hbVisu);
#endif

  m_btnApply->setEnabled(false);
  m_btnCancel->setEnabled(false);

  m_DirSessions=mobilesDir;
  m_DirMobiles=mobilesDir;

  QLabel *lbl=new QLabel(statusBar(),"LblErrors");
  lbl->text()="Erreurs:";
  statusBar()->addWidget(lbl);
  statusBar()->message("Pret",3000);

/*
  // Test des boutons
  for (int i=0;i<30;i++)
  addVisuButton(i+2);
*/
}
// ----------------------------------------------------
void CInterface::activeSave(bool v)
{
  m_bSave = v;
  m_btnSaveSession->setEnabled(v);
  m_btnSaveMobile->setEnabled(v);
}
void CInterface::toggleSave()
{
  m_bSave = !m_bSave;
  activeSave(m_bSave);
}
void CInterface::activePresets(bool v)
{
  m_cbMvtPresets->setEnabled(v);
  m_cbCollidePresets->setEnabled(v);
  m_cbBordLPresets->setEnabled(v);
  m_cbBordRPresets->setEnabled(v);
  m_cbBordTPresets->setEnabled(v);
  m_cbBordBPresets->setEnabled(v);
  m_cbStartPresets->setEnabled(v);
  m_bPresets = v;
  updateDisplay();
}
void CInterface::togglePresets()
{
  m_bPresets = !m_bPresets;
  activePresets(m_bPresets);
}
void CInterface::activeHotkeys(bool v)
{
  m_bHotkeys = v;
}
void CInterface::toggleHotkeys()
{
  m_bHotkeys = !m_bHotkeys;
}
// ----------------------------------------------------
void CInterface::initComboIcons()
{
  static bool init=false;
  if (init)
    return;
  init=true;

  m_cbIcons->insertItem("(-1) Capsule");
  CIconFactory::m_dataDir =(char*) m_DirMobiles.ascii();
  for (int i=0;i<CIconFactory::getNbIcons();i++)
    m_cbIcons->insertItem(QString("(%1) ").arg(i)+CIconFactory::getIcon(i)->name());
}
// ----------------------------------------------------
void CInterface::initComboColors()
{
  for (int i=0;i<s_Colors.size();i++)
    m_cbColors->insertItem(QString("%1").arg(s_Colors[i].m_szName));
  m_cbColors->insertItem(QString(MSG_choix));
}
// ----------------------------------------------------
void CInterface::initPreset(t_presets &presets, QComboBox *combobox, const char * fileName)
{
  ifstream f;
  f.open(fileName);
  string name;
  char c;
  char buff[256];

  presets.clear();

  if (f) { // elimine la premiere ligne si commentaires (dont 'version')
    if( f.get() == '#' )
      f.getline(buff,255);
    else
      f.unget();
  }
  while (f)
  {
    // read title
    f >> name;
    // read expr
    if( f )
    {
      presets.push_back(QString());
      combobox->insertItem(QString(name.c_str()));
      do
      {
	c =  f.get();
	if( c != '#' )
	  presets.back() += c;
      }
      while ( c != '#' && f );
      presets.back()=presets.back().stripWhiteSpace();
    }
  }
}
// ----------------------------------------------------
void CInterface::loadMvtPreset(int n)
{
  if( NULL != m_Selected )
  {
    
    QString str=m_sMvtPresets[n];
    //str=str.replace(QRegExp("\\$"),QString("%1").arg(WORLD.getMobileLocalId(WORLD.getPrevMobile(m_Selected))));
    //m_teExprs->append( str );
    m_teExprs->setText( str );
    //    cerr << " load preset " << n  << " " << m_sMvtPresets[n] << endl;
    applyAll();
    updateDisplay();
  }
}
// ----------------------------------------------------
void CInterface::loadCollidePreset(int n)
{
  if( NULL != m_Selected )
  {
    //m_teExprsCollide->append( m_sCollidePresets[n] );
    m_teExprsCollide->setText( m_sCollidePresets[n] );
    applyAll();
    updateDisplay();
  }
}
// ----------------------------------------------------
void CInterface::loadStartPreset(int n)
{
  if( NULL != m_Selected )
  {
    //m_teExprsStart->append( m_sStartPresets[n] );
    m_teExprsStart->setText( m_sStartPresets[n] );
    applyAll();
    updateDisplay();
  }
}
// ----------------------------------------------------
void CInterface::loadBordLPreset(int n)
{
  if( NULL != m_Selected )
  {
    //m_teExprsBL->append( m_sBordLPresets[n] );
    m_teExprsBL->setText( m_sBordLPresets[n] );
    applyAll();
    updateDisplay();
  }
}
// ----------------------------------------------------
void CInterface::loadBordRPreset(int n)
{
  if( NULL != m_Selected )
  {
    //m_teExprsBR->append( m_sBordRPresets[n] );
    m_teExprsBR->setText( m_sBordRPresets[n] );
    applyAll();
    updateDisplay();
  }
}
// ----------------------------------------------------
void CInterface::loadBordTPreset(int n)
{
  if( NULL != m_Selected )
  {
    //m_teExprsBT->append( m_sBordTPresets[n] );
    m_teExprsBT->setText( m_sBordTPresets[n] );
    applyAll();
    updateDisplay();
  }
}
// ----------------------------------------------------
void CInterface::loadBordBPreset(int n)
{
  if( NULL != m_Selected )
  {
    //m_teExprsBB->append( m_sBordBPresets[n] );
    m_teExprsBB->setText( m_sBordBPresets[n] );
    applyAll();
    updateDisplay();
  }
}
// ----------------------------------------------------
void CInterface::addMobile()
{
  applyAll();
  m_Selected=WORLD.addMobile();
  m_cbMobiles->setCurrentItem(m_Selected->getRank());
  updateDisplay();
}
// ----------------------------------------------------
void CInterface::activateMobile(int n)
{
  int id;

  applyAll();

  QString s=m_cbMobiles->text(m_cbMobiles->currentItem());
  if (!s.isEmpty())
  {
    if (sscanf(s.latin1(),"local %d",&id) == 1)
    {
      m_Selected=WORLD.getLocalMobile(id-1);
      m_Selected->setEditing(false);
      updateDisplay();
    }
  }
}
// ----------------------------------------------------
void CInterface::updateDisplay(bool update_exprs)
{
  if (NULL != m_Selected)
    {
      if (update_exprs)
	{
	  // update exprs without generating textChanged !
	  disconnect( m_teExprs, SIGNAL( textChanged() ), 0,0 );
	  disconnect( m_teExprsBL, SIGNAL( textChanged() ), 0,0 );
	  disconnect( m_teExprsBR, SIGNAL( textChanged() ), 0,0 );
	  disconnect( m_teExprsBT, SIGNAL( textChanged() ), 0,0 );
	  disconnect( m_teExprsBB, SIGNAL( textChanged() ), 0,0 );
	  disconnect( m_teExprsStart, SIGNAL( textChanged() ), 0,0 );
	  disconnect( m_teExprsCollide, SIGNAL( textChanged() ), 0,0 );
	  m_teExprs->setText(QString(m_Selected->getExprs().c_str()));
	  m_teExprsCollide->setText(QString(m_Selected->getExprsCollide().c_str()));
	  m_teExprsStart->setText(QString(m_Selected->getExprsStart().c_str()));
	  m_teExprsBL->setText(QString(m_Selected->getExprsBorder(WORLD_BORDER_LEFT).c_str()));
	  m_teExprsBR->setText(QString(m_Selected->getExprsBorder(WORLD_BORDER_RIGHT).c_str()));
	  m_teExprsBB->setText(QString(m_Selected->getExprsBorder(WORLD_BORDER_BOTTOM).c_str()));
	  m_teExprsBT->setText(QString(m_Selected->getExprsBorder(WORLD_BORDER_TOP).c_str()));
	  connect( m_teExprs, SIGNAL( textChanged() ), this, SLOT( exprsChanged() ) );
	  connect( m_teExprsBL, SIGNAL( textChanged() ), this, SLOT( exprsChanged() ) );
	  connect( m_teExprsBR, SIGNAL( textChanged() ), this, SLOT( exprsChanged() ) );
	  connect( m_teExprsBT, SIGNAL( textChanged() ), this, SLOT( exprsChanged() ) );
	  connect( m_teExprsBB, SIGNAL( textChanged() ), this, SLOT( exprsChanged() ) );
	  connect( m_teExprsStart, SIGNAL( textChanged() ), this, SLOT( exprsChanged() ) );
	  connect( m_teExprsCollide, SIGNAL( textChanged() ), this, SLOT( exprsChanged() ) );
	}
      // icons
      m_cbIcons->setCurrentItem(m_Selected->getIcon()+1);
      // error ?
      if (m_Selected->isValid())
	m_lblError->hide();
      else
	m_lblError->show();
      if (m_Selected->isStopped())
	{
	  m_btnStop->setText("Start !");
	  m_lblRunning->hide();
	  m_lblStopped->show();
	}
      else
	{
	  m_btnStop->setText("Stop !");
	  m_lblRunning->show();
	  m_lblStopped->hide();
	}
      // force refresh
      refreshDisplay(true);
      // disable / enable edition
      m_leX->setEnabled(m_Selected->isStopped());
      m_leY->setEnabled(m_Selected->isStopped());
      m_leDx->setEnabled(m_Selected->isStopped());
      m_leDy->setEnabled(m_Selected->isStopped());
      m_leAgl->setEnabled(m_Selected->isStopped());
      m_leWidth->setEnabled(m_Selected->isStopped());
      m_leHeight->setEnabled(m_Selected->isStopped());
      m_cbIcons->setEnabled(m_Selected->isStopped());
      m_cbColors->setEnabled(m_Selected->isStopped());
      m_teExprs->setEnabled(true);
      m_teExprsCollide->setEnabled(true);
      m_teExprsStart->setEnabled(true);
      m_teExprsBL->setEnabled(true);
      m_teExprsBR->setEnabled(true);
      m_teExprsBB->setEnabled(true);
      m_teExprsBT->setEnabled(true);
      m_btnApply->setEnabled(m_Selected->isEdited());
      m_btnCancel->setEnabled(m_Selected->isEdited());
    }
  else
    {
      m_teExprs->setText("");
      m_teExprsCollide->setText("");
      m_teExprsStart->setText("");
      m_teExprsBL->setText("");
      m_teExprsBR->setText("");
      m_teExprsBB->setText("");
      m_teExprsBT->setText("");
      m_leX->setText("");
      m_leY->setText("");
      m_leDx->setText("");
      m_leDy->setText("");
      m_leAgl->setText("");
      m_leWidth->setText("");
      m_leHeight->setText("");
      m_leX->setEnabled(false);
      m_leY->setEnabled(false);
      m_leDx->setEnabled(false);
      m_leDy->setEnabled(false);
      m_leAgl->setEnabled(false);
      m_leWidth->setEnabled(false);
      m_leHeight->setEnabled(false);
      m_cbIcons->setEnabled(false);
      m_cbIcons->setCurrentItem(0);
      m_cbColors->setEnabled(false);
      m_cbColors->setCurrentItem(0);
      m_cbExport->setChecked(false);
      m_cbVisible->setChecked(false);
      //    m_btnColor->hide();
      //    m_cbColors->hide();
      m_lblError->hide();
      m_lblRunning->hide();
      m_lblStopped->hide();
      m_teExprs->setEnabled(false);
      m_teExprsCollide->setEnabled(false);
      m_teExprsStart->setEnabled(false);
      m_teExprsBL->setEnabled(false);
      m_teExprsBR->setEnabled(false);
      m_teExprsBB->setEnabled(false);
      m_teExprsBT->setEnabled(false);
      m_btnApply->setEnabled(false);
      m_btnCancel->setEnabled(false);
    }
  setExprsPaperColor();
}
// ----------------------------------------------------
void CInterface::refreshDisplay(bool force)
{
  if (NULL == m_Selected)
    return;
  if (m_Selected->isStopped() && !force)
    return;
  m_leX->setText(QString("%1").arg(m_Selected->getPos().x(),0,'f',1));
  m_leY->setText(QString("%1").arg(m_Selected->getPos().y(),0,'f',1));
  m_leDx->setText(QString("%1").arg(m_Selected->getSpeed().x(),0,'f',1));
  m_leDy->setText(QString("%1").arg(m_Selected->getSpeed().y(),0,'f',1));
  m_leAgl->setText(QString("%1").arg(m_Selected->getAngle(),0,'f',2));
  m_leWidth->setText(QString("%1").arg(m_Selected->getWidth(),0,'f',1));
  m_leHeight->setText(QString("%1").arg(m_Selected->getHeight(),0,'f',1));
  m_cbExport->setChecked(m_Selected->isExported());
  m_cbVisible->setChecked(m_Selected->isVisible());
  // export
  QString str(" ");
  int     n=0,e=0;
  for (CWorld::it_mobile M=WORLD.getMobiles().begin();
       M!=WORLD.getMobiles().end();M++)
  {
    if ((*M)->isMaster())
    {
      n++;
      if ((*M)->isExported())
      {
	e++;
	str+=QString("%1 ").arg(n);
      }
    }
  }
  if (0 == e)
    m_lblExport->setText(MSG_aucun);
  else if (e == n)
    m_lblExport->setText(MSG_tous);
  else
    m_lblExport->setText(str);
  // display color
  int c;
  for (c=0;c<s_Colors.size();c++)
  {
    if (s_Colors[c].m_iClr == m_Selected->getColor())
    {
      m_cbColors->setCurrentItem(c);
      break;
    }
  }
  if (c == s_Colors.size())
    // custom color
    m_cbColors->setCurrentItem(s_Colors.size());
  // color sample
  c=m_Selected->getColor();
  QColor clr(((c >> 24)& 255),
	     ((c >> 16)& 255),
	     ((c >>  8)& 255));
  m_fColor->setPaletteBackgroundColor(clr);
  // icon
  m_cbIcons->setCurrentItem(m_Selected->getIcon()+1);
}
// ----------------------------------------------------
void CInterface::applyAll()
{
  setValues();
  applyExprs();
  m_ViewPort->setFocus();
}
// ----------------------------------------------------
void CInterface::applyExprs()
{
  if (NULL == m_Selected)
    return;
  m_Selected->setEditing(false);
  // insert line return
  if (!m_teExprs->text().contains(QRegExp("\n$")))
      m_teExprs->setText(m_teExprs->text()+"\n");
  if (!m_teExprsCollide->text().contains(QRegExp("\n$")))
      m_teExprsCollide->setText(m_teExprsCollide->text()+"\n");
  if (!m_teExprsStart->text().contains(QRegExp("\n$")))
      m_teExprsStart->setText(m_teExprsStart->text()+"\n");
  if (!m_teExprsBL->text().contains(QRegExp("\n$")))
      m_teExprsBL->setText(m_teExprsBL->text()+"\n");
  if (!m_teExprsBR->text().contains(QRegExp("\n$")))
      m_teExprsBR->setText(m_teExprsBR->text()+"\n");
  if (!m_teExprsBB->text().contains(QRegExp("\n$")))
      m_teExprsBB->setText(m_teExprsBB->text()+"\n");
  if (!m_teExprsBT->text().contains(QRegExp("\n$")))
      m_teExprsBT->setText(m_teExprsBT->text()+"\n");
  // update mobile
  m_Selected->setExprs(m_teExprs->text().latin1());
  m_Selected->setExprsCollide(m_teExprsCollide->text().latin1());
  m_Selected->setExprsStart(m_teExprsStart->text().latin1());
  m_Selected->setExprsBorder(WORLD_BORDER_LEFT,m_teExprsBL->text().latin1());
  m_Selected->setExprsBorder(WORLD_BORDER_RIGHT,m_teExprsBR->text().latin1());
  m_Selected->setExprsBorder(WORLD_BORDER_BOTTOM,m_teExprsBB->text().latin1());
  m_Selected->setExprsBorder(WORLD_BORDER_TOP,m_teExprsBT->text().latin1());
  m_Selected->compile();
  // applied
  m_btnApply->setEnabled(false);
  m_btnCancel->setEnabled(false);
  m_Selected->setEditing(false); // need to be done a second time because of QT signals
}
// ----------------------------------------------------
void CInterface::cancelExprs()
{
  if (NULL != m_Selected)
    m_Selected->setEditing(false);
  updateDisplay();
  m_btnApply->setEnabled(false);
  m_btnCancel->setEnabled(false);
}
// ----------------------------------------------------
void CInterface::exprsChanged()
{
  if (NULL != m_Selected)
    m_Selected->setEditing(true);
  m_btnApply->setEnabled(true);
  m_btnCancel->setEnabled(true);
}
// ----------------------------------------------------
void CInterface::changeColor(int n)
{
  QColor clr;
  int r=0,g=0,b=0,c=0;
  
  if (NULL == m_Selected)
    return;
  
  if (n == s_Colors.size())
  {
    clr=QColorDialog::getColor(QColor(r,g,b));
    if (clr.isValid())
    {
      clr.rgb(&r,&g,&b);
      c+=(((int)r) & 255) << 24;
      c+=(((int)g) & 255) << 16;
      c+=(((int)b) & 255) <<  8;
      m_Selected->setColor(c);
    }
    else
    {
      refreshDisplay();
      return;
    }
  }
  else
  {
    c=s_Colors[n].m_iClr;
    m_Selected->setColor(c);
    clr=QColor(((c >> 24)& 255),
	       ((c >> 16)& 255),
	       ((c >>  8)& 255));
  }
  m_fColor->setPaletteBackgroundColor(clr);

}
// ----------------------------------------------------
void CInterface::startStopSelected()
{
  if (m_Selected != NULL)
  {
    // apply all changes
    applyAll();
    // start / stop
    m_Selected->setStopped(!m_Selected->isStopped());
    // display updated information
    updateDisplay(false);
 }
}
// ----------------------------------------------------
void CInterface::setValues()
{
  if (m_Selected == NULL)
    return;

  // set values if mobile is stopped
  if (m_Selected->isStopped())
  {
    // -> set values
    m_Selected->pos().setX(m_leX->text().toDouble());
    m_Selected->pos().setY(m_leY->text().toDouble());
    m_Selected->speed().setX(m_leDx->text().toDouble());
    m_Selected->speed().setY(m_leDy->text().toDouble());
    m_Selected->setAngle(m_leAgl->text().toDouble());
    m_Selected->setWidth(m_leWidth->text().toDouble());
    m_Selected->setHeight(m_leHeight->text().toDouble());
  }
}
// ----------------------------------------------------
void CInterface::cloneSelected()
{
  applyAll();
  if (m_Selected != NULL)
  {
    WORLD.addMobile(m_Selected=new CMobile(m_Selected));
    m_cbMobiles->setCurrentItem(m_Selected->getRank());
    updateDisplay();
    m_Selected->updateExport();
    m_Selected->updateVisible();
  }
}
// ----------------------------------------------------
void CInterface::load()
{
  #ifdef QT3_SUPPORT
	QFileDialog::Options options = 0;
	options |= QFileDialog::DontUseSheet;
	QString s = QFileDialog::getOpenFileName( this, "Choose a file to open", m_DirSessions, "Mobiles (*.mobile)", 0, options);
  #else
	QString s = QFileDialog::getOpenFileName( m_DirMobiles,"Mobiles (*.mobile)",this,"open file dialog", "Choose a file" );
  #endif
  applyAll();
  if (!s.isEmpty())
    {
      m_DirMobiles=extractPath(s);
      // load
	  cout<<"passe ici?"<<endl;
	  m_Selected=PERSIST_LOAD_INSTANCE(CMobile,s.latin1());
      if (m_Selected != NULL)
      {
        m_Selected->setRank(-1);
	      WORLD.addMobile(m_Selected);
	      m_cbMobiles->setCurrentItem(m_Selected->getRank());
	      m_Selected->compile();
	      m_Selected->updateExport();
	      m_Selected->updateVisible();
      }
      updateDisplay();
    }
}
// ----------------------------------------------------
void CInterface::saveSelected()
{
  if (m_Selected == NULL)
    return;
  applyAll();
  QString s = QFileDialog::getSaveFileName( m_DirMobiles,
					    "Mobiles (*.mobile)",
					    this,
					    "open file dialog"
					    "Choose a file" );
  if (!s.isEmpty())
    {
      m_DirMobiles=extractPath(s);
      // save
      if (!s.contains(QRegExp(".mobile$")))
	s=s+".mobile";
      try
	{
	  PERSIST_SAVE_INSTANCE(m_Selected,s.latin1());
	}
      catch (CException& e)
	{
	  QMessageBox::warning(this,"Mobinet",MSG_err_save+s,"OK");
	}
    }
}
// ----------------------------------------------------
void CInterface::loadSession()
{
  #ifdef QT3_SUPPORT
	QFileDialog::Options options = 0;
	options |= QFileDialog::DontUseSheet;
	QString s = QFileDialog::getOpenFileName( this, "Choose a file to open", m_DirSessions, "Sessions (*.session)", 0, options);
  #else
	QString s = QFileDialog::getOpenFileName( m_DirMobiles,"Sessions (*.session)",this,"open file dialog", "Choose a file" );
  #endif

  applyAll();
  if (!s.isEmpty())
    {
      // rest previous session
      resetSession();
      // mem path
      m_DirSessions=extractPath(s);
      // load
	  try {
		CPersistFactory::getInstance()->xmlLoad(s.latin1());
	  }
	  catch(double fileVersion)
	  {
		  cout<<"warning : file version is "<<fileVersion<<" but mobinet script version is "<<PERSIST_FACTORY_MAJOR<<"."<<PERSIST_FACTORY_MINOR<<endl;
		  QMessageBox yesnoBox(this);
		  QAbstractButton *yesButton = yesnoBox.addButton(QMessageBox::Yes);
		  QAbstractButton *noButton = yesnoBox.addButton(QMessageBox::No);
		  yesnoBox.setText("WARNING :\n \n The version of the session you want to load is more recent than the current mobinet script version.\n \n It is possible that it doesn't work well on your version of MobiNet.\n \n Do you really want to load this session?");

		  yesnoBox.exec();
		  if (yesnoBox.clickedButton() == yesButton) {
			CPersistFactory::getInstance()->xmlLoad(s.latin1(), false);
		  }
	  }
      // add loaded mobiles
      for (CPersistFactory::iterator I=PERSIST.begin();
	   I!=PERSIST.end();I++)
	{
	  CMobile *m=dynamic_cast<CMobile *>(*I);
	  if (m != NULL)
	    {
	      WORLD.addMobile(m);
	      m->compile();
	      m->updateExport();
	      m->updateVisible();
	    }
	}
      activateMobile(m_cbMobiles->currentItem());
      updateDisplay();
    }
}

// ----------------------------------------------------
void CInterface::loadSession(QString &s)
{
  applyAll();
  if (!s.isEmpty())
    {
      // rest previous session
      resetSession();
      // mem path
      m_DirSessions=extractPath(s);
      // load
      PERSIST_LOAD(s.latin1());
      // add loaded mobiles
      for (CPersistFactory::iterator I=PERSIST.begin();
	   I!=PERSIST.end();I++)
	{
	  CMobile *m=dynamic_cast<CMobile *>(*I);
	  if (m != NULL)
	    {
	      WORLD.addMobile(m);
	      m->compile();
	      m->updateExport();
	      m->updateVisible();
	    }
	}
      activateMobile(m_cbMobiles->currentItem());
      updateDisplay();
    }
}

// ----------------------------------------------------
void CInterface::saveSession()
{
  applyAll();
  QString s = QFileDialog::getSaveFileName( m_DirSessions,
					    "Sessions (*.session)",
					    this,
					    "open file dialog"
					    "Choose a file" );
  if (!s.isEmpty())
    {
      m_DirSessions=extractPath(s);
      // save
      if (!s.contains(QRegExp(".session$")))
	s=s+".session";
      try
	{
	  PERSIST_SAVE(s.latin1());
	}
      catch (CException& e)
	{
	  QMessageBox::warning(this,"Mobinet",MSG_err_save+s,"OK");
	}
    }
}
// ----------------------------------------------------
void CInterface::supprSelected()
{
  if (m_Selected == NULL)
    return;
  WORLD.removeMobile(m_Selected);
  m_Selected=NULL;
  //m_cbMobiles->changeItem("(deleted)",m_cbMobiles->currentItem());
  m_cbMobiles->removeItem(m_cbMobiles->currentItem());
  activateMobile(m_cbMobiles->currentItem());
  updateDisplay();
}
// ----------------------------------------------------
void CInterface::exportChanged()
{
  if (m_Selected == NULL)
    return;
  m_Selected->setExport(m_cbExport->isChecked());
  m_Selected->updateExport();
}
// ----------------------------------------------------
void CInterface::visibleChanged()
{
  if (m_Selected == NULL)
    return;
  m_Selected->setVisible(m_cbVisible->isChecked());
  m_Selected->updateVisible();
}
// ----------------------------------------------------
void CInterface::iconChange(int n)
{
  if (m_Selected == NULL)
    return;
  m_Selected->setIcon((n-1) % CIconFactory::getNbIcons());
}
// ----------------------------------------------------
void CInterface::showEvent(QShowEvent *)
{
  initComboIcons();
}
// ----------------------------------------------------
void CInterface::resetSelected()
{
  if (m_Selected != NULL)
  {
    m_Selected->reset();
    updateDisplay();
    refreshDisplay();
  }
}
// ----------------------------------------------------
void CInterface::resetSession()
{
  WORLD.reset();
  // WORLD.resetTime();
  m_Selected=NULL;
  m_cbMobiles->clear();
  g_MainWindow->m_ViewPort->resetDisplay();
  updateDisplay();
}
// ----------------------------------------------------
void CInterface::wheelEvent(QWheelEvent *e)
{
  if (focusWidget() == m_leX)
    m_leX->setText(QString("%1").arg(m_leX->text().toDouble() + 1.0*e->delta()/WHEEL_DELTA,0,'f',1));
  else if (focusWidget() == m_leY)
    m_leY->setText(QString("%1").arg(m_leY->text().toDouble() + 1.0*e->delta()/WHEEL_DELTA,0,'f',1));
  else if (focusWidget() == m_leDx)
    m_leDx->setText(QString("%1").arg(m_leDx->text().toDouble() + 0.1*e->delta()/WHEEL_DELTA,0,'f',1));
  else if (focusWidget() == m_leDy)
    m_leDy->setText(QString("%1").arg(m_leDy->text().toDouble() + 0.1*e->delta()/WHEEL_DELTA,0,'f',1));
  else if (focusWidget() == m_leAgl)
    m_leAgl->setText(QString("%1").arg(m_leAgl->text().toDouble() + 5.0*(M_PI/180.0)*e->delta()/WHEEL_DELTA,0,'f',2));
  else if (focusWidget() == m_leWidth)
    m_leWidth->setText(QString("%1").arg(m_leWidth->text().toDouble() + 0.2*e->delta()/WHEEL_DELTA,0,'f',1));
  else if (focusWidget() == m_leHeight)
    m_leHeight->setText(QString("%1").arg(m_leHeight->text().toDouble() + 0.5*e->delta()/WHEEL_DELTA,0,'f',1));
  else
  {
    e->ignore();
    return;
  }
  setValues();
}
// ----------------------------------------------------
void CInterface::viewAll()
{
  for (std::map<int,QPushButton *>::iterator B=m_Buttons.begin();
       B!=m_Buttons.end();B++)
  {
    QPushButton *btn=(*B).second;
    if (NULL != btn)
    {
      if (btn->isEnabled())
	btn->setOn(true);
     }
  }
  updateDisplay();
}
// ----------------------------------------------------
void CInterface::viewNone()
{
  for (std::map<int,QPushButton *>::iterator B=m_Buttons.begin();
       B!=m_Buttons.end();B++)
  {
    QPushButton *btn=(*B).second;
    if (NULL != btn)
    {
      if (btn->isEnabled())
	btn->setOn(false);
     }
  }
  updateDisplay();
}
// ----------------------------------------------------
void CInterface::exportAll()
{
  for (CWorld::it_mobile M=WORLD.getMobiles().begin();M!=WORLD.getMobiles().end();M++)
  {
    if ((*M)->isMaster())
    {
      (*M)->setExport(true);
      (*M)->updateExport();
    }
  }
  updateDisplay();
}
// ----------------------------------------------------
void CInterface::exportNone()
{
  for (CWorld::it_mobile M=WORLD.getMobiles().begin();M!=WORLD.getMobiles().end();M++)
  {
    if ((*M)->isMaster())
    {
      (*M)->setExport(false);
      (*M)->updateExport();
    }
  }
  updateDisplay();
}
// ----------------------------------------------------
void CInterface::selectPrev()
{
  if (m_cbMobiles->count() == 0)
    return;
  m_cbMobiles->setCurrentItem((m_cbMobiles->currentItem()-1+m_cbMobiles->count()) % m_cbMobiles->count());
  activateMobile(m_cbMobiles->currentItem());
  updateDisplay();
}
// ----------------------------------------------------
void CInterface::selectNext()
{
  if (m_cbMobiles->count() == 0)
    return;
  m_cbMobiles->setCurrentItem((m_cbMobiles->currentItem()+1) % m_cbMobiles->count());
  activateMobile(m_cbMobiles->currentItem());
  updateDisplay();
}
// ----------------------------------------------------
void CInterface::addVisuButton(int cli,bool enabled)
{
  QPushButton *btn;
  //cerr << "Adding button " << cli << endl;
#ifdef QT3_SUPPORT
  btn = new QPushButton("btn"+QString("%1").arg(cli));
  m_hbVisu->addWidget(btn, Qt::AlignLeft);
#else
  btn = new QPushButton("btn"+QString("%1").arg(cli), m_hbVisu);
#endif
  btn->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed));
  btn->setGeometry( QRect( 10+(cli % 16)*25, 10+(cli/16)*25, 20, 20 ) ); 
  btn->setFixedWidth(20);
  btn->setFixedHeight(20);
  btn->setText(QString("%1").arg(cli+1));
  btn->setToggleButton(true);
  m_Buttons[cli]=btn;
  btn->show();
  btn->setEnabled(enabled);
}
// ----------------------------------------------------
void CInterface::removeVisuButton(int cli)
{
  //cerr << "Removing button " << (cli+1) << endl;
  std::map<int,QPushButton *>::iterator B=m_Buttons.find(cli);
  if (B != m_Buttons.end())
  {
    QPushButton *btn=(*B).second;
    m_Buttons.erase(B);
    btn->hide();
    delete (btn);
  }
}
// ----------------------------------------------------
bool CInterface::isClientVisible(int cli)
{
  std::map<int,QPushButton *>::iterator B=m_Buttons.find(cli);
  if (B != m_Buttons.end())
  {
    QPushButton *btn=(*B).second;
    return (btn->isOn() || !btn->isEnabled());
  }
  return (false);
}
// ----------------------------------------------------
void CInterface::setExprsPaperColor()
{
  static QColor blackColor(0,0,0);
  static QColor whiteColor(255,255,255); 
  static QColor redColor(255,160,160);

  if (m_Selected != NULL)
    if (m_Selected->isEdited())
      return;
  m_teExprs->setPaper(QBrush(whiteColor));
  m_teExprsCollide->setPaper(QBrush(whiteColor));
  m_teExprsStart->setPaper(QBrush(whiteColor));
  m_teExprsBB->setPaper(QBrush(whiteColor));
  m_teExprsBR->setPaper(QBrush(whiteColor));
  m_teExprsBT->setPaper(QBrush(whiteColor));
  m_teExprsBL->setPaper(QBrush(whiteColor));
  m_teExprs->setColor(blackColor);
  m_teExprsCollide->setColor(blackColor);
  m_teExprsStart->setColor(blackColor);
  m_teExprsBB->setColor(blackColor);
  m_teExprsBR->setColor(blackColor);
  m_teExprsBT->setColor(blackColor);
  m_teExprsBL->setColor(blackColor);
  if (m_Selected != NULL)
    {
      if (!m_Selected->isValid())
      {
        QTextEdit *te=NULL;
        switch (m_Selected->getExprLastError())
        {
        case 0 : te=g_MainWindow->m_teExprs;
          break;
        case 1 : te=g_MainWindow->m_teExprsStart;
          break;
        case 2 : te=g_MainWindow->m_teExprsCollide;
          break;
        case 3+WORLD_BORDER_BOTTOM : te=g_MainWindow->m_teExprsBB;
          break;
        case 3+WORLD_BORDER_RIGHT : te=g_MainWindow->m_teExprsBR;
          break;
        case 3+WORLD_BORDER_TOP : te=g_MainWindow->m_teExprsBT;
          break;
        case 3+WORLD_BORDER_LEFT : te=g_MainWindow->m_teExprsBL;
          break;
        default: 
          break;
        }
        if (te != NULL)
        {
            te->setPaper(QBrush(redColor));
            te->setFocus();
            int l=m_Selected->getExprLineError();
            if (l > 0)
            {
              te->moveCursor(QTextEdit::MoveHome,false);
              for (int i=1;i<l;i++)
                te->moveCursor(QTextEdit::MoveDown,false);
              te->moveCursor(QTextEdit::MoveDown,true);
            }
            m_Selected->setEditing(true);
        }
      }
    }
}
// ----------------------------------------------------
QString CInterface::extractPath(const QString& str)
{
  int p=str.findRev('/');
  if (p < 0)
    return (QString("."));
  else
    {
      QString s=str;
      s.truncate(p);
      return (s);
    }
}
// ----------------------------------------------------
